% Proving set containment with the S-procedure
% Use the S-procedure to prove that the set S2 is contained
% in the set S1 where
%     S1 := { [x;y] : g1(x,y)<= 0}
%     S2 := { [x;y] : g2(x,y)<= 0}
pvar x y;
g1 = 5*y^4+x^2*y^2 - y^3 + 3*x^2*y +4*x^2 - 7;
g2 = 4*x^2-x*y+9*y^2-2;

% The S-procedure sufficient condition:  
% If there exists s(x) such that s(x) is SOS
% and  -g1+s*g2 is SOS then S2 is contained in S1.

% Create the multiplier decision variable
sdegs = 0:1;
smonom = monomials([x;y],sdegs);
s = polydecvar('scoef',smonom,'mat');

% Store the SOS constraints in a cell array
cons{1} = s;
cons{2} = -g1+s*g2;

% Solve the SOS feasibility problem
[info,dopt,sossol]=sosopt(cons,[x;y])

% Plot the level sets of S1 and S2 to verify the set containment
figure(1);
plotaxes = [-2 2 -2 2];
pcontour(g1,0,plotaxes,'b'); hold on;
pcontour(g2,0,plotaxes,'r');

% Use pcontain to maximize gamma subject to:
% S_{2,gamma} := { [x;y] : g2(x,y)<= gamma} is contained in S1.
[gbnds,sout]=pcontain(g1,g2,smonom)
pcontour(g2,gbnds(1),plotaxes,'g'); 
legend('g1=0','g2=0','g2=gamma')