
% Set up system
e = 0.1;
a = 1.1;

pvar x1 x2;
f = [ 1/e*( x1- (1/3)*(x1-a)^3 - x2 - a^3/3 ); x1+0.01*x2];
x = [x1;x2];
f = cleanpoly(f,1e-5);

% Create phase plane
for r = [1.5 5];
    Npts = 50;
    th = linspace(0,2*pi,Npts);
    x0 = r*[sin(th); cos(th)];
    Tf = 10;
    xtraj = psim(f,x,x0,Tf); 
    for i1=1:Npts
        xx=xtraj{i1,2};
        plot(xx(:,1),xx(:,2)); hold on;
    end
end
axis([-4 4 -4 4]);

% Define decision variable for quadratic Lyapunov function
zV = monomials(x,2:10);
V = polydecvar('c',zV,'vec');

% Constraint 1 : V(x) - L1 \in SOS
L1 = 1e-6*(x1^2+x2^2);
sosconstr = cell(2,1);
sosconstr{1} = V-L1;

% Constraint 2: -Vdot - L2 \in SOS
% (PJS: For quadratic V, one can show that it is not possible to force
%       -Vdot-L2 \in SOS and we must settle for -Vdot \in SOS]
Vdot = jacobian(V,x)*f;
L2 = 1e-6*(x1^2+x2^2);
L2=0;
sosconstr{2} =-Vdot-L2;

% Solve with feasibility problem
opts.fid = 1;
%opts.form = 'kernel';
opts.form = 'image';
opts.checkfeas = 'off';
[info,dopt,sossol] = sosopt(sosconstr,x,opts);
if ~isempty(dopt)
    Vsol = subs(V,dopt)
    %ls = [1/16 1/4 1/2 0.65 0.8 1 2 4]/1e5;
    ls = [1/32 1/16 1/8 1/4 1/2 1 2]/1e3;
    %ls = linspace(1/3,1,50)/1e3;
    pcontour(Vsol,ls,[-4 4 -4 4],'r--',200*[1 1]);
end

%-------------------Iteration
return
% Create shape function and vector of Lyapunov function monomials
Vdeg = 6;
p = x'*x;
zV = monomials(x,2:Vdeg);    
z1maxd = ceil((Vdeg-p.maxdeg)/2);
%z1 = monomials(x, 0:z1maxd ); 
z1 = monomials(x, 0:2 ); 
z2 = monomials(x, 1:2 );
L2 = 1e-6*(x'*x);

% Construct Lyap function from linearization 
Vlin=linstab(f,x);

% Run V-s iteration
Nsteps = 30; 
ph = [];
opts = [];
opts.gmin = 0; 
opts.gmax = 10;
opts.L2 = L2;
%aviobj = avifile('example.avi','fps',2);
for i1=1:Nsteps;    
    if i1==1
        V = Vlin;        
    else
        % roavstep
        [V,c] = roavstep(f,p,x,zV,b,g,s1,s2,opts);
        
        % Scale V by previous gamma.  This roughly normalizes V
        V = V/g;
    end
    
    % get gamma
    [gbnds,s2]=pcontain(jacobian(V,x)*f+L2,V,z2,opts);
    g = gbnds(1);

    % get beta
    [bbnds,s1]=pcontain(V-g,p,z1,opts);
    b = bbnds(1);
    
    fprintf('i1 = %d  \t beta = %4.3f\n',i1,b);

    % Plot V/p contours for ROA estimate 
    if  ~isempty(ph)
        delete(ph)
    end
    domain = [-3 3 -3 3]/5;
    [C,ph(1)]=pcontour(V,g,domain,'r');
    [C,ph(2)]=pcontour(p,b,domain,'g');
    title(['Iteration = ' int2str(i1) ' beta = ' num2str(b)]);
    
    %F(i1) = getframe(gcf);
    %aviobj = addframe(aviobj,F(i1));
    drawnow;    
end
%aviobj = close(aviobj);


return;
%-------------------
% Check that V is decreasing on an interesting trajectory
x0 = [2.6; 0.55];
xtraj = psim(f,x,x0,Tf);
tt=xtraj{1}; xx=xtraj{2};
V2 = subs(Vsol,{x1,x2},{xx(:,1),xx(:,2)});
V2 = double(V2);
plot(tt,V2);
