%------------------------------------------------------------------
% VDP with VS Iteration Initialized with Vlin
%   Compute an estimate of the region of attraction for the van 
%   der Pol oscillator using the V-S iteration.  The iteration is 
%   initialized with the Lyapunov function from linearized analysis.
%------------------------------------------------------------------
clear all; close all;

% Create vector field
pvar x1 x2;
x = [x1;x2];
x1dot = -x2;
x2dot = x1+(x1^2-1)*x2;
f = [x1dot; x2dot];

% Plot VDP limit cycle from backward simulation
X0 = [1/2; 1/2];
Tf = 100;
[xtraj,isconv]=psim(-f,x,X0,Tf);
t = xtraj{1};
xtraj = xtraj{2};
idx = min(find(t>=Tf*0.8));

figure(1)
hold off;
plot(xtraj(idx:end,1),xtraj(idx:end,2));
xlabel('x1');
ylabel('x2');
hold on;


% Create shape function and vector of Lyapunov function monomials
Vdeg = 6;
p = x'*x;
zV = monomials(x,2:Vdeg);    
z1maxd = ceil((Vdeg-p.maxdeg)/2);
z1 = monomials(x, 0:z1maxd ); 
z2 = monomials(x, 1:2 );
L2 = 1e-6*(x'*x);

% Construct Lyap function from linearization 
Vlin=linstab(f,x);

% Run V-s iteration
Nsteps = 5; 
ph = [];
opts = [];
opts.gmin = 0; 
opts.gmax = 10;
opts.L2 = L2;
%aviobj = avifile('example.avi','fps',2);
for i1=1:Nsteps;    
    if i1==1
        V = Vlin;        
    else
        % roavstep
        [V,c] = roavstep(f,p,x,zV,b,g,s1,s2,opts);
        
        % Scale V by previous gamma.  This roughly normalizes V
        V = V/g;
    end
    
    % get gamma
    [gbnds,s2]=pcontain(jacobian(V,x)*f+L2,V,z2,opts);
    g = gbnds(1);

    % get beta
    [bbnds,s1]=pcontain(V-g,p,z1,opts);
    b = bbnds(1);
    
    fprintf('i1 = %d  \t beta = %4.3f\n',i1,b);

    % Plot V/p contours for ROA estimate 
    if  ~isempty(ph)
        delete(ph)
    end
    domain = [-3 3 -3 3];
    [C,ph(1)]=pcontour(V,g,domain,'r');
    [C,ph(2)]=pcontour(p,b,domain,'b');
    title(['Iteration = ' int2str(i1) ' beta = ' num2str(b)]);
    
    %F(i1) = getframe(gcf);
    %aviobj = addframe(aviobj,F(i1));
    drawnow;    
end
%aviobj = close(aviobj);

% Call PenBMI
ic.beta = b;
ic.gamma = g;
ic.s1 = s1;
ic.s2 = s2;
ic.V = V;
[bopt,gopt,s1opt,s2opt,Vopt] = roabmi(f,x,p,zV,opts,ic);

set(ph,'LineStyle',':');
[C,ph(1)]=pcontour(Vopt,gopt,domain,'r');
[C,ph(2)]=pcontour(p,bopt,domain,'b');
set(ph,'LineWidth',2)
title(['PENBMI:  beta = ' num2str(bopt)]);
